      program fp
c------------------------------------------------------------
c This program is a template for postprocessing of output   
c data.
c
c It checks the current directory for all files of the type 
c d_l<lambda>.<k>, where <lambda> and <k> is an integer numbers, 
c and <lambda> is between lambda_min and lambda_max. 
c 
c Then, for each <lambda>, the program reads the dipole file with 
c extension <k> and calculates the intensity of electric field, |E|^2
c at an arbitrary point in space. The output is written into a file 
c as a number of lines of the form
c         lambda     |E|^2
c          .............
c
c All necessary parameters are read by the program interactively from
c the keyboard.

      character*32 name_out, name_coord, name_dip
      real lattice_unit, lambda
      real pol(3), prop(3)
      integer x(10 000), y(10 000), z(10 000)
      complex dx(10 000), dy(10 000), dz(10 000)
      complex dxc, dyc, dzc
      complex cu, cu3, ce, acfc, bcfc
      complex ex, ey, ez, exc, eyc, ezc, sprod
      complex arg, inc_phase
      character*1 propchar, polchar, lchar
      logical exc_pw, exist_flag

 1    format(a32)
      pi = 4.*atan(1.)
      cu = (0.,1.)
      cu3 = (0.,1.)
c      
c------ Interactive input --------------------------------------------
c
      write(*,*) 'Give radius [nm]' ! The value of parameter radius in nm
      read(*,*) radius
      lattice_unit = radius * (4.*pi / 3.)**(1./3.)

      write(*,*) 'Give excitation type; enter 1 for plane waves'
      write(*,*) 'or any other number for evanescent field'
      read(*,*) index
      if(index .eq. 1) then
        exc_pw = .true.
        write(*,*) 'Assuming excitation by plane wave'
      else
        exc_pw = .false.
        write(*,*) 'Assuming excitation by evanescent field'
      end if

      if(.not. exc_pw) then
 20      continue
         write(*,*) 'Give refractive index of the prizm'
         read(*,*) refractive_index   ! The value of the parameter 
                                      ! refractice_index
         write(*,*) 'Give angle of incidence in units of pi'
         read(*,*) theta_fraction_pi  ! The value of the parameter
                                      ! theta_fraction_pi
         theta = theta_fraction_pi * pi
         if(refractive_index * sin(theta) .le. 1) then
           write(*,*) 'No internal reflection with these of parameters'
           write(*,*) 'Enter the same parameters as were used in'
           write(*,*) 'the calculations... Try again'
           goto 20
         end if
      end if


 17   continue
      prop(1) = 0.
      prop(2) = 0.
      prop(3) = 0.
      pol(1)  = 0.
      pol(2)  = 0.
      pol(3)  = 0.

      if(exc_pw) then
 14     continue
        write(*,*) 'Give propagation direction [x,y,z]'
        read(*,18) propchar
 18     format(a1)
        if(propchar .eq. 'x') then 
          prop(1) = 1.
          goto 13
        end if
        if(propchar .eq. 'y') then 
          prop(2) = 1.
          goto 13
        end if
        if(propchar .eq. 'z') then 
          prop(3) = 1.
          goto 13
        end if
        write(*,*) 'Propagation direction is incorrect; please repeat.'
        goto 14
 13     continue

 16     continue
        write(*,*) 'Give polarization direction [x,y,z]'
        read(*,18) polchar
        if(polchar .eq. 'x') then
          pol(1) = 1.
          goto 15
        end if
        if(polchar .eq. 'y') then
          pol(2) = 1.
          goto 15
        end if
        if(polchar .eq. 'z') then
          pol(3) = 1.
          goto 15
        end if
        write(*,*) 'Polarization direction is incorrect; please repeat.'
        goto 16
 15     continue

        scalar_product = pol(1)*prop(1)+pol(2)*prop(2)+pol(3)*prop(3)
        if(scalar_product .gt. 1e-15) then
          write(*,*) 'Pol and prop are not orthogonal; reenter.'
          goto 17
        end if
      else
        prop(1) = refractive_index * sin(theta)
        prop(2) = 0.
        prop(3) = sqrt(prop(1)**2 - 1.)
        pol(1) = 0.
        pol(2) = 1.
        pol(3) = 0.
      end if

      write(*,*) 'Give coordinates of the point,' 
      write(*,*) '[real, in lattice units]'
      read(*,*) xtip, ytip, ztip ! The point where |E|^2 is to be calculated

 21   continue
      write(*,*) 'Give lambda units; Enter'
      write(*,*) 'm for microns, n for nanometers a for Angstroms'
      read(*,18) lchar
      if(lchar .ne. 'm' .and. lchar .ne. 'n' .and. lchar .ne. 'a') then
         write(*,*) 'Your input is not understood; retry'
         goto 21
      end if
      if(lchar .eq. 'm') fact_lambda = 1000.
      if(lchar .eq. 'n') fact_lambda = 1.0
      if(lchar .eq. 'a') fact_lambda = 0.1

 22   continue
      write(*,*) 'Give lambda_min, lambda_max,'
      write(*,*) '[integer, in lambda_units]'
      read(*,*) lambda_min, lambda_max
      if(lambda_min .gt. lambda_max) then
         write(*,*) 'lambda_max must be greater than lambda_min;'
         write(*,*) 'reenter'
         goto 22
      end if

      write(*,*) 'Give the dipole files extension [integer number]'
      read(*,*) next

      goto 6
 5    continue
      write(*,*) 'The file was not found; retry.'
 6    continue
      write(*,*) 'Give input filename with coordinates of dipoles'
      read(*,1) name_coord ! The value of the parameter name_coordinates

      open(unit=70, file=name_coord, status='old', err=5)
      i = 0
 7    continue
      read(70,*,err=8,end=8) ix, iy, iz
      i = i + 1
      x(i) = ix
      y(i) = iy
      z(i) = iz
      goto 7
 8    continue
      n = i
      close(unit=70)
      write(*,*) 'N=', n

c-------- Opening output file
      write(*,*) 'Give output filename'
      read(*,1) name_out ! Any output filename
      open(unit=60, file=name_out, status='unknown')

c---------- Loop over lambda
      do 2, il = lambda_min,lambda_max

c-------- Constructing and checking filenames
      ndec1 = log10(float(il)) + 1
      ndec2 = log10(float(next)) + 1
      write(name_dip, 4) il, next
 4    format('d_l', i<ndec1>, '.', i<ndec2>)
      inquire(file=name_dip, exist=exist_flag)
      if(.not. exist_flag) goto 2
      lambda = float(il) * fact_lambda ! lambda converted to nanometers

c--------- Setting constants for given lambda
      wave_vector = 2.*pi/lambda
      wave_vector3 = wave_vector**3
      const = wave_vector * lattice_unit
      const2 = const**2

c--------- Finding the phase of the incident field at the point
      if(exc_pw) then
        arg = prop(1)*xtip + prop(2)*ytip + prop(3)*ztip
      else 
        arg = prop(1)*xtip + prop(2)*ytip + prop(3)*ztip*cu
      end if
      inc_phase = cexp(const*cu*arg)

c-------- Opening and reading the dipole file for given lambda
      open(unit=71, file=name_dip, status='old', 
     $     access='direct', recl=24)
      n = 0
 10   continue
      n = n + 1
      read(71, rec=n, end=11, err=11) dxc, dyc, dzc
      dx(n) = dxc
      dy(n) = dyc
      dz(n) = dzc
      goto 10
 11   continue
      close(unit=71)
      n = n - 1

c------- Initializing summation of the secondary scattered fields
c                        at the point
      ex = (0.,0.)
      ey = (0.,0.)
      ez = (0.,0.)

c------- Loop over dipoles
      do 12, i=1,n
      xit = xtip - x(i)
      yit = ytip - y(i)
      zit = ztip - z(i)
      sprod = (xit*dx(i) + yit*dy(i) + zit*dz(i))
      drit2 = 1. / ((xit**2 + yit**2 + zit**2)*const2)
      drit  = sqrt(drit2)
      drit3 = drit**3
      rit = 1./drit
      ce = cexp(cu*rit)
      acfc = wave_vector3 * ( drit + cu* drit2 -    drit3)*ce
      bcfc = wave_vector3 * (-drit - cu3*drit2 + 3.*drit3)*ce
      exc = acfc*dx(i) + bcfc*sprod*xit*const2*drit2
      eyc = acfc*dy(i) + bcfc*sprod*yit*const2*drit2
      ezc = acfc*dz(i) + bcfc*sprod*zit*const2*drit2
      ex = ex + exc
      ey = ey + eyc
      ez = ez + ezc
 12   continue
c-------- Summation of the scattered fields completed

c-------- The total field at the point is a sum of the secondary
c         (scattered) field and the incident field
      ex = ex + pol(1)*inc_phase
      ey = ey + pol(2)*inc_phase
      ez = ez + pol(3)*inc_phase
      
c-------- Calculating and writing the local intensity
      e2 = ex*conjg(ex) + ey*conjg(ey) + ez*conjg(ez)
      write(60, *) lambda, e2
      write(*, *) 'lambda=', lambda, ' [nm];  |E|^2=', e2

 2    continue

 3    continue
      close(unit=60)
      close(unit=70)

      stop
      end


