C23456789012345678901234567890123456789012345678901234567890123456789012
C------ The above line is 72 characters long. Do not exceed it -------->
      program lu

      real lambda_min, lambda_max
      character*(*) lambda_units
      real minimum_distance
      character*(*) dip_dir, dipres_dir, q_dir
      character*(*) file_extension, name_coordinates
      character*(*) name_epsilon
      logical excitation_evanescent_field_par
      logical excitation_plane_wave_par
      logical report_time, report_progress
      logical finite_size_effects
      logical always_use_drude, always_use_other
      logical append_efficiency_data
      logical execute_precise
      complex cu, cu3
c      
c----- The adjustable parameters follow this line listed by group ------
c
c-- Dimensions of arrays
      parameter (namelength = 100)
      parameter (nmon=1000)

c-- Defining the incident wave
      parameter (excitation_evanescent_field_par = .false.) 
      parameter (excitation_plane_wave_par = .true.)        
      parameter (polx=1., poly=0., polz=0.)
      parameter (propx=0., propy=0., propz=1.)
      parameter (refractive_index = 1.52)
      parameter (theta_fraction_pi = 0.25)

c-- Controlling lambda's
      parameter (lambda_min=1000., lambda_max=1000.) 
      parameter (lambda_units = 'nanometers')
           ! allowed values: 'angstroms', 'nanometers', 'microns'
      parameter (nl_par = 1)

c-- I/O directories and filenames
      parameter (dip_dir = './')
      parameter (dipres_dir='./')
      parameter (file_extension = '.1')
      parameter (name_coordinates = './f.1')
      parameter (name_epsilon = './johnson.dat')
      parameter (q_dir = './')

c-- Assignement of polarizability, alpha
      parameter (always_use_drude = .false.)  
      parameter (always_use_other = .false.)
      parameter (finite_size_effects = .false.)
      parameter (radius = 5.)  ! In nanometers

c-- Reporting time and progress
      parameter (report_progress = .true.)
      parameter (report_time = .true.)

c-- Miscellaneous
      parameter (append_efficiency_data = .true.)
      parameter (execute_precise = .false.)
      parameter (minimum_distance = 0.9)

c----------- Do not change the parameters below this line --------------
      parameter ( cu = (0.,1.), cu3=(0.,3.) )
      parameter (nmon3 = 3*nmon)

      integer*2 x(nmon), y(nmon), z(nmon)
      complex a(nmon3*(nmon3+1)/2)
      integer*2 alfa, beta, dr(3)
      real prop(3), pol(3)
      complex arg
      complex ce, ki, dki, aij, bij, aii
      complex zet, dzet, dzet2, dzete
      complex sum1, sum2, tii
      character*(namelength) name_dip, name_dipres, name_q
      character*(namelength) name_dip_pp
      character*(namelength) name_dipres_pp
      character*(namelength) name_q_pp
      character*1 propchar, polchar
      character*8 ctime, cdate
      real lattice_unit, lambda
      complex rhp(nmon3), d(nmon3), d1(nmon3)

      logical flag, exist_flag1, exist_flag2, exist_flag3
      logical       exist_flag4, exist_flag5, exist_flag_pp
      logical excitation_evanescent_field, excitation_plane_wave
      
      external mkdircc
c
c-------------------------------------------------------------------
c
      if(report_time) then
         call time(ctime)
         call date(cdate)
         write(*,*) '---------------------------------------------------
     $---------'
         write(*,*) '---------------------------------------------------
     $---------'
         write(*,*) 'Beginning the LU expansion program: ', 
     $              cdate, '  ', ctime
         write(*,*) '---------------------------------------------------
     $---------'
         write(*,*) '---------------------------------------------------
     $---------'
      end if
c
c------------ Initial settings -------------------------------------
c
      pi = 4.*datan(1.d0)
      lattice_unit = radius * (4.*pi / 3.)**(1./3.)
      l = nmon3*(nmon3+1)/2
 2    format(a<namelength>)
c
c--------- Checking excitation field type
c
      excitation_evanescent_field = excitation_evanescent_field_par
      excitation_plane_wave       = excitation_plane_wave_par
      if(excitation_evanescent_field .eqv. excitation_plane_wave) then
         write(*,*) 'Warning: excitation type was not selected properly'
         if(execute_precise) goto 300
         write(*,*) 'Assuming excitation by a plane wave'
         excitation_evanescent_field = .false.
         excitation_plane_wave = .true.
      end if
      if(excitation_evanescent_field) then
         theta = pi * theta_fraction_pi
         if(refractive_index*sin(theta) .le. 1.) then
            write(*,*) 'Evanescent field excitation was chosen;'
            write(*,*) 'however, with the selected parameters'
            write(*,*) 'refractive_index=', refractive_index
            write(*,*) 'theta=', theta_fraction_pi, '*pi'
            write(*,*) 'there is no complete internal reflection.'
            if(execute_precise) goto 300
            write(*,*) 'Excitation by a plane wave will be'
            write(*,*) 'assumed in this calculation.'
            excitation_evanescent_field = .false.
            excitation_plane_wave = .true.
         end if
      end if
c
c-------- Checking prescription type for polarizability ------------
c
      if(always_use_drude .and. always_use_other) then
         write(*,*) 'Warning: both always_use_drude and
     $always_use_other were set to .true.'
         if(execute_precise) goto 300
         write(*,*) 'The Drude formula will be used.'
      end if
c
c-------- Checking directories -----------------------------------------
c
      i_dip_dir = len(dip_dir)
      i_q_dir = len(q_dir)   
      i_dipres_dir = len(dipres_dir)
      i_file_extension = len(file_extension)
      i_name_coordinates = len(name_coordinates)
      i_name_epsilon = len(name_epsilon)

      inquire(file=dip_dir, exist=exist_flag1)
      if(.not. exist_flag1) then
         write(*,*) 'Fatal incompatibility of input encountered:'
         write(*,*) 'the directory to store dipole moments'
         write(*,90) dip_dir
 90      format(a<i_dip_dir>, ' does not exist or is unaccessable.')
         write(*,*) 'Create the directory or change parameter 
     $ dip_dir; exiting.'
         goto 300
      end if

      inquire(file=dipres_dir, exist=exist_flag2)
      if(.not. exist_flag2) then
         write(*,*) 'Non-fatal incompatibility of input encountered:'
         write(*,*) 'Reserve directory to store dipole moments'
         write(*,91) dipres_dir
 91      format(a<i_dipres_dir>, ' does not exist or is unaccessable.')
         if(execute_precise) goto 300
         write(*,*) 'Data can be lost; proceed at your own risk'
         write(*,*) 'or stop and create the directory or change 
     $ parameter dipres_dir'
      end if

      inquire(file=name_coordinates, exist=exist_flag4)
      if(.not. exist_flag4) then
         write(*,*) 'Fatal incompatibility of input encountered:'
         write(*,*) 'the input file with coordinates'
         write(*,98) name_coordinates
 98      format(a<i_name_coordinates>, ' does not exist or is 
     $unaccessable')
         write(*,*) 'Create it or change parameter name_coordinates; 
     $exiting.'
         goto 300
      end if

      inquire(file=q_dir, exist=exist_flag3)
      if(.not. exist_flag3) then
         write(*,*) 'Fatal incompatibility of input encountered:'
         write(*,*) 'the directory to store Qe, Qa and Qs '
         write(*,94) q_dir
 94      format(a<i_q_dir>, ' is not found or is unaccessable;')
         write(*,*) 'Exiting.'
         goto 300
      end if

      inquire(file=name_epsilon, exist=exist_flag5)
      if(.not. exist_flag5) then
         write(*,*) 'Warning: the input file with epsilon data '
         write(*,99) name_epsilon
 99      format(a<i_name_epsilon>, ' is not found or is unaccessable.')
         if((.not. always_use_drude) .and. 
     $      (.not. always_use_other)) then
            write(*,*) 'This file is required by the program.'
            if(execute_precise) goto 300
            write(*,*) 'But Drude formula will be used instead.'
         end if
      end if
c
c--------- Checking compatibility of lambda_min, lambda_max and nl
c
      nl = nl_par
      if(nl .le. 0) then
         write(*,*) 'Warning: unacceptable value: nl=', nl
         if(execute_precise) goto 300
         write(*,*) 'Assuming nl=1 in this calculation'
         nl = 1
      end if
      if(nl .eq. 1 .and. lambda_min .ne. lambda_max) then
         write(*,*) 'Warning: nl=1 but lambda_min =/= lambda_max'
         if(execute_precise) goto 300
         write(*,*) 'Assuming lambda = lambda_min in this calculation;'
         write(*,*) 'lambda_max ignored'
      end if
      if(nl .gt. 1 .and. lambda_min .eq. lambda_max) then
         write(*,*) 'Warning: nl>1 but lambda_min = lambda_max.'
         if(execute_precise) goto 300
         write(*,*) 'Assuming nl=1 and lambda = lambda_min in this 
     $ calculation;'
         write(*,*) 'lambda_max ignored.'
         nl = 1
      end if

      write(*,*) 'Lambda units: ', lambda_units
      if(lambda_units .eq. 'angstroms') then
         fact_lambda = 0.1
      else if(lambda_units .eq. 'nanometers') then
         fact_lambda = 1.0
      else if(lambda_units .eq. 'microns') then
         fact_lambda = 1000.
      else
         write(*,*) 'Illegal lambda_units'
         if(execute_precise) goto 300
         write(*,*) 'nanometers will be assumed for the lambda_units.'
         fact_lambda = 1.
      end if
c
c---------- Seting propchar and polchar (to be used for
c                  constructing the output filenames)
c
      propchar = 'u'
      polchar = 'u'

      if(excitation_evanescent_field) then
         propchar = 'e'
         polchar = 'e'
         write(*,*) 'Assuming excitation by evanescent field.'
         write(*,*) 'Parameters prop and pol are discarded.'
         write(*,*) 'Light falling at the angle', theta_fraction_pi, 
     $              '*pi to the internal surface.'
         write(*,*) 'Refractive index of the material:', 
     $              refractive_index
         theta = pi * theta_fraction_pi
         prop(1) = refractive_index * sin(theta)
         prop(2) = 0.
         prop(3) = sqrt(prop(1)**2 - 1.)
         pol(1) = 0.
         pol(2) = 1.
         pol(3) = 0.
      end if

      if(excitation_plane_wave) then
         write(*,*) 'Assuming excitation by a plane wave'
         prop(1) = propx
         prop(2) = propy
         prop(3) = propz
         prop_mod = sqrt(prop(1)**2 + prop(2)**2 + prop(3)**2)
         if(prop_mod .ne. 1.) then
            write(*,*) 'Warning: prop_mod .ne. 1.'
            if(execute_precise) goto 300
            prop(1) = prop(1)/prop_mod
            prop(2) = prop(2)/prop_mod
            prop(3) = prop(3)/prop_mod
         end if
         if(abs(prop(1)-1.) .lt. 1e-7) propchar='x'
         if(abs(prop(2)-1.) .lt. 1e-7) propchar='y'
         if(abs(prop(3)-1.) .lt. 1e-7) propchar='z'
         pol(1) = polx
         pol(2) = poly
         pol(3) = polz       ! Light polarization direction
         pol_mod = sqrt(pol(1)**2 + pol(2)**2 + pol(3)**2)
         if(pol_mod .ne. 1.) then
            write(*,*) 'Warning: pol_mod .ne. 1.'
            if(execute_precise) goto 300
            pol(1) = pol(1)/pol_mod
            pol(2) = pol(2)/pol_mod
            pol(3) = pol(3)/pol_mod
         end if
         if(abs(pol(1)-1.) .lt. 1e-7) polchar='x'
         if(abs(pol(2)-1.) .lt. 1e-7) polchar='y'
         if(abs(pol(3)-1.) .lt. 1e-7) polchar='z'
      end if

      if(propchar .eq. 'u' .or. polchar .eq. 'u') then
         write(*,*) 'Warning: either propagation or polarization'
         write(*,*) 'direction does not coinside with x-y-z axes.'
         write(*,*) 'The ''unknown'' char (''u'') is assigned'
         write(*,*) 'in the corresponding position of the output path.'
      end if
c
c------ Creating output subdirectories
c
cc-- Comment the following lines if you do not want to use mkdircc
      write(name_dip_pp, 170) dip_dir, propchar, polchar
 170  format(a<i_dip_dir>, 'prop=', a1, '_pol=', a1)
      inquire(file=name_dip_pp, exist=exist_flag_pp)
      if(.not. exist_flag_pp) then
         call mkdircc(name_dip_pp)
         write(*,201) name_dip_pp
 201     format('Directory ', a<i_dip_dir+12>, ' created')
      end if

      write(name_dipres_pp, 171) dipres_dir, propchar, polchar
 171  format(a<i_dipres_dir>, 'prop=', a1, '_pol=', a1)
      inquire(file=name_dipres_pp, exist=exist_flag_pp)
      if(.not. exist_flag_pp) then
         call mkdircc(name_dipres_pp)
         write(*,202) name_dipres_pp
 202     format('Directory ', a<i_dipres_dir+12>, ' created')
      end if

      write(name_q_pp, 172) q_dir, propchar, polchar
 172  format(a<i_q_dir>, 'prop=', a1, '_pol=', a1)
      inquire(file=name_q_pp, exist=exist_flag_pp)
      if(.not. exist_flag_pp) then
         call mkdircc(name_q_pp)
         write(*,203) name_q_pp
 203     format('Directory ', a<i_q_dir+12>, ' created')
      end if
cc-- Stop commenting here (if you decided not to use mkdircc)
c
c-------------- Checking that k E_0 = 0 --------------------------------
c
      sclprod = prop(1)*pol(1) + prop(2)*pol(2) + prop(3)*pol(3)
      if(sclprod .gt. 1.e-10) then
            write(*,*) 'Error: k and E are not orthogonal.'
            if(execute_precise) goto 300
            write(*,*) 'Either stop and correct input or continue'
            write(*,*) 'at your own risk'
      end if            
c
c------------- Checking the prescription for polarizability
c
      if(always_use_drude .and. always_use_other) then
         write(*,*) 'Warning: both always_use_drude and'
         write(*,*) 'always_use_other were set to .true.'
         if(execute_precise) goto 300
         write(*,*) 'The drude formula will be used.'
      end if
c
c---------- Reading dipole coordinates from a file -----------
c
      open(unit=70, file=name_coordinates, status='old', err=96)
      goto 97
 96   continue
      write(*,*) 'File with coordinates ',
     $           name_coordinates, ' couldnt be open.'
      write(*,*) 'Please check directories or parameter 
     $ name_coordinates; exiting.'
      goto 300
 97   continue
      
      n = 0
 3    continue
      read(70,*,end=4) ix, iy, iz
      n = n + 1
      if(excitation_evanescent_field .and. iz .lt. 0) then
         write(*,*) 'Warning: if excitation by evanescent field is'
         write(*,*) 'used, all z coordinates must be nonnegative,'
         write(*,*) 'but z(', i, ')=', iz
         if(execute_precise) goto 300
      end if
      x(n) = ix
      y(n) = iy
      z(n) = iz
      goto 3
 4    continue
      close(unit=70)
      write(*,*) 'Number of particles in the cluster:', n
      n3 = 3*n
      if(n .gt. nmon) then
         write(*,*) 'Fatal incompatibility: parameter nmon is less'
         write(*,*) 'than the number of monomers in the cluster,', n
         write(*,*) 'Correct parameter nmon; exiting'
         goto 300
      else if(n .lt. nmon) then
         write(*,*) 'Warning: parameter nmon is greater'
         write(*,*) 'than the number of monomers in the cluster,', n
         write(*,*) 'Correct parameter nmon to save memory;'
         write(*,*) 'continuing'
      end if
c
c-----------------------------------------------------------
c
      write(name_q, 72) q_dir, propchar, polchar, file_extension
 72   format(a<i_q_dir>, 'prop=', a1, '_pol=', a1, '/q', 
     $       a<i_file_extension>)
      write(*,75) name_q
 75   format('name_q=', a<namelength>)

      write(*,*) 'Beginning cycle over lambda'
         write(*,*) '---------------------------------------------------
     $---------'
         write(*,*) '---------------------------------------------------
     $---------'
c
c------------- Loop over lambda ---------------------------
      do 200, il=1,nl

      if(nl .eq. 1) then
         lambda = lambda_min
      else if(nl .gt. 1) then
         lambda = lambda_min + 
     $            (lambda_max-lambda_min)*float(il-1)/float(nl-1)
      else
         write(*,*) 'Unexpected fatal incompatibility; check code'
         goto 300
      end if
      llambda = lambda + 0.5  ! Closest integer to lambda in lambda_unit
      ndec = alog10(lambda) + 1 ! Number of digits in integer llambda
      lambda = lambda*fact_lambda ! lambda converted to nanometers


      wave_vector = 2.*pi/lambda
      cons1 = lattice_unit*wave_vector
      cons2 = (wave_vector)**3
      cons3 = lattice_unit**3
c
c------- Constructing output filenames
c
      write(name_dip, 70) dip_dir, propchar, polchar, 
     $   llambda, file_extension
 70   format(a<i_dip_dir>, 'prop=', a1, '_pol=', a1, '/d_l', 
     $   i<ndec>, a<i_file_extension>)

      write(name_dipres, 71) dipres_dir, propchar, polchar, 
     $   llambda, file_extension
 71   format(a<i_dipres_dir>, 'prop=', a1, '_pol=', a1, '/d_l', 
     $   i<ndec>, a<i_file_extension>)

      if(report_time) then
         call time(ctime)
         call date(cdate)
         write(*,*) '*********** lambda=', lambda, '[nm] **** ', 
     $              cdate, '  ', ctime, ' ****'
      else
         write(*,*) '*************** lambda=', lambda, 
     $              '[nm] ***************'
      end if
      write(*,73) name_dip
      write(*,74) name_dipres
 73   format('name_dip=',     a<namelength>)
 74   format('name_dipres=',  a<namelength>)
c
c-------- Calling a subroutine to calculate alpha
c      
      if(always_use_drude) then
         write(*,*) 'Using Drude formula'
         call sus_drude(lambda, radius, ki)
      else if(always_use_other) then
         write(*,*) 'Using <other> formula'
         call sus_other(lambda, radius, ki)
      else if(exist_flag5) then
         write(*,*) 'Reading epsilon data from file:', name_epsilon
         if(finite_size_effects) then
            call sus_eps(name_epsilon, lambda, radius, ki, flag)
         else
            call sus_eps0(name_epsilon, lambda, radius, ki, flag)
         end if
         if(flag) then
            write(*,*) 'lambda out of range for the datafile specified'
            write(*,*) 'or the epsilon-data file unreadable.'
            if(execute_precise) goto 300
            write(*,*) 'Using the Drude formula.'
            call sus_drude(lambda, radius, ki)
         else
            write(*,*) 'File ', name_epsilon, ' is OK'
         end if
      else
         write(*,*) 'File ', name_epsilon, ' does not exist;'
         if(execute_precise) goto 300
         write(*,*) 'Using Drude formula.'
         call sus_drude(lambda, radius, ki)
      end if

      dki = 1./ki
      write(*,*) '-Re[1/alpha]=', -cons3*Real(dki),';   -Im[1/alpha]=', 
     $            -cons3*Aimag(dki), ' [lattice units]'
      yabs = -Aimag(dki) - 2.*(wave_vector**3)/3.
      write(*,*) 'alpha/radius^3=', ki/radius**3
      write(*,*) 'y_a/(2k^3/3)=', 3.*yabs/(2.*wave_vector**3)
c------------------------------------------------- end --------------



c-------------- Building elements of interaction matrix A -----
      if(report_time) then
         call time(ctime)
         call date(cdate)
         write(*,*) 'Building elements of matrix: ', cdate, '  ', ctime
      end if
      
      do 101, i=1,n-1
      ii = 3*(i-1)
      do 101, j=i+1,n
      jj = 3*(j-1)
      dr(1) = x(i) - x(j)
      dr(2) = y(i) - y(j)
      dr(3) = z(i) - z(j)
      rij2 = float(dr(1)**2 + dr(2)**2 + dr(3)**2)
      rij = sqrt(rij2)
      if(rij .lt. minimum_distance) then
         write(*,*) 'Warning: incompatibility of the coordinates file:'
         write(*,*) 'Two particles closer than', minimum_distance
         if(execute_precise .or. rij .eq. 0.) goto 300
      end if
      zet = cons1*rij
      dzet = 1./zet
      dzet2 = dzet*dzet
      ce = cexp(cu*zet)
      dzete = dzet*ce*cons2
      aij = -dzete*(1. + cu*dzet - dzet2)
      bij = -dzete*(-1. - cu3*dzet + 3.*dzet2)/rij2

      do 102, alfa=1,3
      iia = ii + alfa
      do 102, beta =1,3
      jjb = jj + beta
      k = jjb*(jjb-1)/2 + iia
      a(k) = bij*dr(alfa)*dr(beta)
      if(alfa .eq. beta) a(k)=a(k) + aij
 102  continue

 101  continue

      do 103, i=1,n
      ii = 3*(i-1)

      do 104, alfa=1,3
      iia = ii + alfa
      k = iia*(iia+1)/2
      a(k) = dki
 104  continue
      
      do 105, alfa=1,2
      iia = ii + alfa
      do 105, beta=alfa+1,3
      iib = ii + beta
      k = iib*(iib-1)/2 + iia
      a(k) = (0.,0.)
 105  continue

 103  continue

      if(report_time) then
         call time(ctime)
         call date(cdate)
         write(*,*) 'Elements of matrix built: ', cdate, '  ', ctime
      end if

c-----------------------------------------------------------------
c------------ Building right-hand part of the equation --------------
      do 9, i=1,n
      if(excitation_evanescent_field) then
         arg = x(i)*prop(1) + y(i)*prop(2) + (z(i)+0.5)*prop(3)*cu
      else if(excitation_plane_wave) then
         arg = x(i)*prop(1) + y(i)*prop(2) + z(i)*prop(3)
      else
         write(*,*) 'Unexpected fatal incompatibility; check code'
         goto 300
      end if
      arg = arg*cons1
      ce = cexp(cu*arg)
      ii = 3*(i-1)
      do 109, alfa=1,3
      rhp(ii+alfa) = pol(alfa)*ce
 109  continue
 9    continue

c\\\\\\\
c---------- Calculation of elements of matrix T (stored in a(k) ---
c///////

      if(report_progress) i_progress = 0
      do 110, i=1,n3
      im1 = i-1
      ip1 = i+1
      kiip = i*ip1/2
      kiim = i*im1/2
      aii = a(kiip)

      sum1 = (0.,0.)
      if(i .le. 1) goto 115
      do 114, l=1,im1
      k1 = l + kiim
      sum1 = sum1 + a(k1)*a(k1)
 114  continue
 115  continue

      tii = csqrt(aii - sum1)

      do 111, j=i,n3
      kjjm = j*(j-1)/2
      kij = i + kjjm
      aij = a(kij)

      sum2 = (0.,0.)
      if(i .le. 1) goto 113
      do 112, l=1,im1
      k1 = kiim + l
      k2 = kjjm + l
      sum2 = sum2 + a(k2)*a(k1)
 112  continue
 113  continue

      a(kij) = (aij - sum2)/tii
 111  continue
      if(report_progress) then
         prog_fract = float(i)/float(n3)
         i_progress = 100 * prog_fract**2 *(3. - 2.*prog_fract)
         if(i_progress .gt. i_progress_old) then
            call time(ctime)
            write(*,*) 'Completed: ', i_progress, '% ; ', ctime
         end if
         i_progress_old = i_progress
      end if
 110  continue
      if(report_time) then
         call time(ctime)
         call date(cdate)
         write(*,*) 'Matrix T built; ', cdate, '  ', ctime
      end if
c\\\\\\\
c------------- Finding solution -------------
c///////
      do 120, i=1,n3
      im1 = i - 1
      ip1 = i + 1
      kiip = i*ip1/2 
      kiim = i*im1/2 
      aii = a(kiip)

      sum1 = (0.,0.)
      if(i .le. 1) goto 122
      do 121, j=1,im1
      k = j + kiim
      sum1 = sum1 + a(k)*d1(j)
 121  continue
 122  continue

      d1(i) = (rhp(i) - sum1)/aii
 120  continue

c:::::::::::::::::::::::::::::::::::::::::

      do 130, i=n3,1,-1
      im1 = i - 1
      ip1 = i + 1
      kiip = i*ip1/2 
      kiim = i*im1/2 
      aii = a(kiip)

      sum1 = (0.,0.)
      if(i .ge. n3) goto 132
      do 131, j=i+1,n3
      jm1 = j - 1
      kjjm = j*jm1/2
      k = kjjm + i
      sum1 = sum1 + a(k)*d(j)
 131  continue
 132  continue
      
      d(i) = (d1(i) - sum1)/aii
 130  continue
      

c\\\\\\\
c------------- Evaluation of the cross-sections -------------
c///////
      sum_e = 0.
      sum_a = 0.
      do 50, i=1,n3
      sum_e = sum_e + aimag( d(i) * conjg(rhp(i)) )
      sum_a = sum_a + d(i)*conjg(d(i))
 50   continue
      qe = 4.*wave_vector*sum_e/(float(n)*radius**2)
      qa = 4.*wave_vector*yabs*sum_a/(float(n)*radius**2)
      qs = qe - qa

      call time(ctime)
      call date(cdate)
      write(*,*) cdate, '   ', ctime, '; qe=', qe
      
      if((il .eq. 1) .and. (.not. append_efficiency_data)) then
         open(unit=70, file=name_q, status='unknown')
      else
         open(unit=70, file=name_q, access='append', status='unknown')
      end if
      write(70,*) lambda, qe, qa, qs
      close(unit=70)
c\\\\\\\\\\\\\
c----------- writing dipole moments ------------------
      open(unit=70, file=name_dip, status='unknown', 
     $     access='direct', recl=24, err=81)
      do 80, i=1,n
      ii = 3*(i-1)
      write(70, rec=i, err=81) d(ii+1), d(ii+2), d(ii+3)
 80   continue
      close(unit=70)
      index_write = 1

      goto 82
 81   continue

      open(unit=70, file=name_dipres, status='unknown', 
     $     access='direct', recl=24, err=84)
      do 83, i=1,n
      ii = 3*(i-1)
      write(70, rec=i, err=84) d(ii+1), d(ii+2), d(ii+3)
 83   continue
      close(unit=70)
      index_write = 2
      
      goto 82
 84   continue
      index_write = 0
      write(*,*) '  failing to write dipoles file'
 82   continue
c      write(*,*) 'index_write=', index_write
c\\\\\\\\\\\\\
c------------ Checking discrepancy ----------------

      do 141, i=1,n-1
      ii = 3*(i-1)
      do 141, j=i+1,n
      jj = 3*(j-1)
      dr(1) = x(i) - x(j)
      dr(2) = y(i) - y(j)
      dr(3) = z(i) - z(j)
      rij2 = float(dr(1)**2 + dr(2)**2 + dr(3)**2)
      rij = sqrt(rij2)
      zet = cons1*rij
      dzet = 1./zet
      dzet2 = dzet*dzet
      ce = cexp(cu*zet)
      dzete = dzet*ce*cons2
      aij = -dzete*(1. + cu*dzet - dzet2)
      bij = -dzete*(-1. - cu3*dzet + 3.*dzet2)/rij2

      do 142, alfa=1,3
      do 142, beta =1,3
      k = (n3+1)*(ii+alfa-1) - (ii+alfa-1)*(ii+alfa)/2 +
     $    3*(j-i) + beta - alfa + 1

      a(k) = bij*dr(alfa)*dr(beta)
      if(alfa .eq. beta) a(k)=a(k) + aij

 142  continue
 141  continue

      do 143, i=1,n
      ii = 3*(i-1)

      do 144, alfa=1,3
      k = (n3+1)*(ii+alfa-1) - (ii+alfa-1)*(ii+alfa)/2 + 1
      a(k) = dki
 144  continue
      
      do 145, alfa=1,2
      do 145, beta=alfa+1,3
      k = (n3+1)*(ii+alfa-1) - (ii+alfa-1)*(ii+alfa)/2 +
     $    + beta - alfa + 1
      a(k) = (0.,0.)
 145  continue

 143  continue


      sum = (0.,0.)
      do 146, i=1,n3
      sum2 = (0.,0.)
      do 147, j=1,n3
      if(i .le. j) then
         i1 = i
         j1 = j
      else
         i1 = j
         j1 = i
      end if
      i11 = i1-1
      k = n3*i11 - i11*i1/2 + j1
      sum2 = sum2 + a(k)*d(j)
 147  continue
      sum = sum + ( cabs(rhp(i) - sum2) )**2
 146  continue
      sum = sum/float(n3)
      write(*,*) 'Discrepancy:', sum

c-----------------------------------------------------------------
 200  continue
c-----------------------------------------------------------------
      goto 301
 300  continue
      write(*,*) 'Either a fatal incompatibility was encountered'
      write(*,*) 'or parameter execute_precise was set to .true.'
      write(*,*) 'and a non-fatal incompatibility has occured.'
      write(*,*) 'Check the program output for warnings.'
 301  continue
      if(report_time) then
         call time(ctime)
         call date(cdate)
         write(*,*) '---------------------------------------------------
     $----------'
         write(*,*) '---------------------------------------------------
     $----------'
         write(*,*) 'Exiting the LU expansion program: ', 
     $               cdate, '  ', ctime
         write(*,*) '---------------------------------------------------
     $----------'
         write(*,*) '---------------------------------------------------
     $----------'
      end if

      stop
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc                                                        ccc
ccc               S U B R O U T I N E S                    ccc
ccc                                                        ccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

        SUBROUTINE SUS_EPS(NAME_EPS, LAMBDA, RADIUS, KI, FLAG)

        COMPLEX KI
        REAL LAMBDA
        CHARACTER*(*) NAME_EPS
        COMPLEX xill, cu, eps
        REAL L, L_OLD, L_MIN, L_MAX, LAMBDA_P, K, K3
        LOGICAL FLAG

        FLAG = .FALSE.
	PI = 4.*ATAN(1.)
	CU = (0.,1.)
	a = radius	! Radius of monomer [nm]
	ag = a*1.4	! Radius of electron confinment
	a2 = a*a	! Radius squared
	a3 = a*a*a	! Radius cubed
	lambda_p = 136.1! Wavelength (of the incident wave) 
			! at the frequency corresponding to 
			! plasmon resonance of silver [nm]
	g_inf = .0019	! Gamma for bulk silver (the constant
			! appearing in the Drude formula) devided
			! by the plasmon frequency for silver
	vf = .0047	! Fermi velocity for silver devided
			! by the velocity of light
	eps_ext = 1.	! dielectric permiability of external
c	eps_ext = 1.78	! media (=1. default). For clusters in
			! water change 1. to 1.78;
c       eps_ext = 1.53  ! In gelatin: change to 1.53

C********************************************************************
        OPEN(UNIT=70, FILE=NAME_EPS, STATUS='OLD', ERR=100)	

        l_min = 10000.
        l_max = 0.
        read(70,*,end=13, err=100) l, r1, r2
        if(l .lt. l_min) l_min = l
        if(l .gt. l_max) l_max = l
        i = 1
        l_old = l
        r1_old = r1
        r2_old = r2

12	continue
	read(70,*,end=13, err=100) l, r1, r2
        if(l .lt. l_min) l_min = l
        if(l .gt. l_max) l_max = l
	i = i + 1
        if(l .eq. lambda) goto 13
        if( (lambda-l_old)*(lambda-l) .le. 0.) then
           r1 = r1_old + (lambda-l_old)*(r1-r1_old)/(l-l_old)
           r2 = r2_old + (lambda-l_old)*(r2-r2_old)/(l-l_old)
           goto 13
        end if
        l_old = l
        r1_old = r1
        r2_old = r2
        goto 12
13      continue
        close (unit=70)
        if(l_min .gt. lambda .or. l_max .lt. lambda) then
           flag = .true.
           goto 101
        end if

        l = lambda
	k = 2.*pi/l
	k3 = k*k*k
	w = (l/lambda_p)
	w2 = w*w
	eps = (r1 + cu*r2)**2
	g_0 = w*g_inf
	g_a = g_0 + vf*(l/ag) / (2.*pi)
	eps = eps + w2*(1./(1. + cu*g_0) - 1./(1. + cu*g_a) )
	eps = eps/eps_ext
	xill = a3*(eps - 1.)/(eps + 2.)
	ki = xill/(1. - cu*(2.*k3/3.)*xill)

        goto 101
 100    CONTINUE
        flag = .true.
 101    continue

        RETURN
        END

c----------------------------------------------------------------

        SUBROUTINE SUS_EPS0(NAME_EPS, LAMBDA, RADIUS, KI, FLAG)

        COMPLEX KI
        REAL LAMBDA
        CHARACTER*(*) NAME_EPS
        COMPLEX xill, cu, eps
        REAL L, L_OLD, L_MIN, L_MAX, K, K3
        LOGICAL FLAG

        FLAG = .FALSE.
	PI = 4.*ATAN(1.)
	CU = (0.,1.)
	a = radius	! Radius of monomer [nm]
	a2 = a*a	! Radius squared
	a3 = a*a*a	! Radius cubed
	eps_ext = 1.	! dielectric permiability of external
c	eps_ext = 1.78	! media (=1. default). For clusters in
			! water change 1. to 1.78
c       eps_ext = 1.53  ! In gelatin: change to 1.53


C********************************************************************
        OPEN(UNIT=70, FILE=NAME_EPS, STATUS='OLD', ERR=100)	

        l_min = 100000.
        l_max = -1.
        read(70,*,end=13, err=100) l, r1, r2
        if(l .lt. l_min) l_min = l
        if(l .gt. l_max) l_max = l
        i = 1
        l_old = l
        r1_old = r1
        r2_old = r2

12	continue
	read(70,*,end=13, err=100) l, r1, r2
        if(l .lt. l_min) l_min = l
        if(l .gt. l_max) l_max = l
	i = i + 1
        if(l .eq. lambda) goto 13
        if( (lambda-l_old)*(lambda-l) .le. 0.) then
           r1 = r1_old + (lambda-l_old)*(r1-r1_old)/(l-l_old)
           r2 = r2_old + (lambda-l_old)*(r2-r2_old)/(l-l_old)
           goto 13
        end if
        l_old = l
        r1_old = r1
        r2_old = r2
        goto 12
13      continue
        close (unit=70)
        if(l_min .gt. lambda .or. l_max .lt. lambda) then
           flag = .true.
           goto 101
        end if

        l = lambda
	k = 2.*pi/l
	k3 = k*k*k
	eps = (r1 + cu*r2)**2
	xill = a3*(eps - 1.)/(eps + 2.)
	ki = xill/(1. - cu*(2.*k3/3.)*xill)

        goto 101
 100    CONTINUE
        flag = .true.

 101    continue
        RETURN
        END

c----------------------------------------------------------------

       SUBROUTINE SUS_DRUDE(LAMBDA, RADIUS, KI)
       REAL LAMBDA, LAMBDA_G, LAMBDA_P
       COMPLEX CU, EPS, KI_LL, KI

       PI = 4.*ATAN(1.)
       CU = (0.,1.)
       A = RADIUS
       EPS_EXT = 1.
       LAMBDA_P = 136.1
       LAMBDA_G = 71631.58
       GAMMA_OVER_OMEGA_P = LAMBDA_P / LAMBDA_G
       WAVE_VECTOR = 2.*PI/LAMBDA

       RL = LAMBDA/LAMBDA_P
       EPS = 1. - RL**2 / (1. + CU * GAMMA_OVER_OMEGA_P * RL)
       EPS = EPS/EPS_EXT
       KI_LL = A**3 * (EPS - 1.) / (EPS + 2.)
       KI = 1./(1./KI_LL - CU*(2./3.)*WAVE_VECTOR**3)

       RETURN
       END

c----------------------------------------------------------------

       SUBROUTINE SUS_OTHER(LAMBDA, RADIUS, KI)
       REAL LAMBDA
       COMPLEX CU, EPS, KI_LL, KI


       PI = 4.*ATAN(1.)
       CU = (0.,1.)
       A = RADIUS
       EPS_EXT = 1.

       WAVE_VECTOR = 2.*PI/LAMBDA
                     !
       EPS = (2.,1.) ! Change EPS or write another code for
                     ! determining EPS here
                     !
       EPS = EPS/EPS_EXT
       KI_LL = A**3 * (EPS - 1.) / (EPS + 2.)
       KI = 1./(1./KI_LL - CU*(2./3.)*WAVE_VECTOR**3)

       RETURN
       END

c----------------------------------------------------------------
